package org.sun.jsat.trees;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;
import securityanalyzer.resourcemodel.FilledActionDefinition;
import securityanalyzer.resourcemodel.ParameterDefinition;

public class FilledActionDefTreeNode extends RefreshableTreeNode<FilledActionDefinition> {

    public FilledActionDefTreeNode(FilledActionDefinition fad) {
        super(fad, fad.getBaseObject().getName());
        this.baseObject = fad;
    }

    @Override
    public void generateMissingChildren() {
        synchronized (updateMutex){
            generationActions();
            for (ParameterDefinition p : baseObject.getAllParameters()){
                if (childrenMap.get(p) == null){
                    FilledParameterDefTreeNode pdtn = new FilledParameterDefTreeNode(p, baseObject.getMapping(p), ((FilledActionDefinition)baseObject).isDefault(p));
                    childrenMap.put(p, pdtn);
                    this.add(pdtn);
                }
            }
        }
    }

    @Override
    public void removeExtraChildren() {
        synchronized (updateMutex){
            generationActions();
            List<ParameterDefinition> pds = new ArrayList<ParameterDefinition>();
            for (Entry<Object, RefreshableTreeNode> e : childrenMap.entrySet()){
                if (e.getKey() instanceof ParameterDefinition){
                    ParameterDefinition p = (ParameterDefinition)e.getKey();
                    if (!baseObject.getBaseObject().getParameters().contains(p)){
                        pds.add(p);
                        this.remove(e.getValue());
                    }
                }
            }
            for(ParameterDefinition pd: pds) {
                childrenMap.remove(pd);
            }
        }
    }
    
    @Override
    public void refresh(){
        synchronized (updateMutex){
            super.refresh();
            for (Entry<Object, RefreshableTreeNode> e : childrenMap.entrySet()){
                ParameterDefinition pd = (ParameterDefinition)e.getKey();
                FilledParameterDefTreeNode pdtn = (FilledParameterDefTreeNode)e.getValue();
                if (pdtn.getValue() == null || !pdtn.getValue().equals(baseObject.getMapping(pd))){
                    pdtn.setValue(baseObject.getMapping(pd));
                }
            }
        }
    }
}
