package org.sun.jsat.trees;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;
import securityanalyzer.resourcemodel.ResourceDefinition;
import securityanalyzer.resourcemodel.ResourceModel;
import securityanalyzer.resourcemodel.RuleDefinition;

/**
 *
 * 
 */
public class ResourceModelRootTreeNode extends RefreshableTreeNode<ResourceModel> {

    public static final int DISPLAY_RESOURCES = 1;
    public static final int DISPLAY_RULES = 2;
    private int displayType;
    
    public ResourceModelRootTreeNode(int displayType) {
        super(null, getTitle(displayType));
        this.displayType = displayType;
    }

    @Override
    public void generateMissingChildren() {
        synchronized (updateMutex) {
            generationActions();
            switch (displayType){
                case DISPLAY_RESOURCES:
                    for (ResourceDefinition rd : ResourceModel.getAllResources()) {
                        if (childrenMap.get(rd) == null) {
                            ResourceDefTreeNode rdtn = new ResourceDefTreeNode(rd);
                            childrenMap.put(rd, rdtn);
                            this.add(rdtn);
                        }
                    }
                    break;
                case DISPLAY_RULES:
                    for (RuleDefinition rd : ResourceModel.getAllRules()) {
                        if (childrenMap.get(rd) == null) {
                            RuleDefTreeNode rdtn = new RuleDefTreeNode(rd);
                            childrenMap.put(rd, rdtn);
                            this.add(rdtn);
                        }
                    }
                    break;
            }
        }
    }
    
    @Override
    public void removeExtraChildren() {
        synchronized (updateMutex){
            generationActions();
            switch (displayType){
                case DISPLAY_RESOURCES:
                    List<ResourceDefinition> rds = new ArrayList<ResourceDefinition>();
                    for (Entry<Object, RefreshableTreeNode> e : childrenMap.entrySet()){
                        if (e.getKey() instanceof ResourceDefinition){
                            ResourceDefinition r = (ResourceDefinition)e.getKey();
                            if (!ResourceModel.getAllResources().contains(r)){
                                rds.add(r);
                                this.remove(e.getValue());
                            }
                        }
                    }
                    for(ResourceDefinition rd: rds) {
                        childrenMap.remove(rd);
                    }
                    break;
                case DISPLAY_RULES:
                    List<RuleDefinition> rdes = new ArrayList<RuleDefinition>();
                    for (Entry<Object, RefreshableTreeNode> e : childrenMap.entrySet()){
                        if (e.getKey() instanceof RuleDefinition){
                            RuleDefinition r = (RuleDefinition)e.getKey();
                            if (!ResourceModel.getAllRules().contains(r)){
                                rdes.add(r);
                                this.remove(e.getValue());
                            }
                        }
                    }
                    for(RuleDefinition rd: rdes) {
                        childrenMap.remove(rd);
                    }
                    break;
            }
        }
    }

    @Override
    public String toolTipText() {
        return "Contains all defined " + getTitle(displayType);
    }
    
    private static String getTitle(int displayType){
        switch (displayType){
            case DISPLAY_RESOURCES:
                return "Resources";
            case DISPLAY_RULES:
                return "Rules";
            default:
                return "??";
        }
    }
}
