package securityanalyzer.analysisexecution;

import java.util.ArrayList;
import java.util.List;

import securityanalyzer.analysismodel.PermissionRequirement;


/**
 * 
 * Describes an analysis tool - something capable of getting the AnalysisModel
 * up to date
 * 
 * Any additional parameters needed to start analysis should enter through the
 * class's constructor
 *
 */
public abstract class AnalysisTool {
	
	protected List<PermissionRequirement> findings = new ArrayList<PermissionRequirement>();

	/** 
	 * The type of analysis this tool performs
	 * @return
	 */
	public abstract String getAnalysisType();

	/**
	 * Set the type of analysis this tool performs
	 * @param analysisType
	 */
	public abstract void setAnalysisType(String analysisType);

	/**
	 * 
	 * Run the analysis specified. Every time a result is found, it should be
	 * added to our list of results using
	 * addPermissionRequirementFind(PermissionRequirement)
	 * 
	 */
	public abstract void runAnalysis();
	
	/**
	 * Add a result to our list of findings
	 */
	public boolean addPermissionRequirementFind(PermissionRequirement pr){
		return findings.add(pr);
	}
	
	/**
	 * Remove a result from our list of findings
	 * @param pr result to remove
	 * @return true if we had the specified finding
	 */
	public boolean removePermissionRequirementFind(PermissionRequirement pr){
		return findings.remove(pr);
	}
	
	/**
	 * Get all the findings of this tool
	 * @return
	 */
	public List<PermissionRequirement> getAllFindings(){
		return findings;
	}
}
