package securityanalyzer.analysismodel;

/**
 * 
 * Describes an invocation of a method, including noting if it's privileged or
 * run in a separate thread
 * 
 */
public class MethodInvocation {

	/** The method performing the invocation */
	private Method invokingMethod;
	/** The method that would be invoked */
	private Method methodCalled;
	/** True if the method call is privileged */
	private boolean priv;
	/** True if the method call is a separate thread */
	private boolean thread;
	
	/**
	 * Constructor
	 * @param m The method that would be invoked  
	 * @param priv True if the method call is privileged
	 * @param thread True if the method call is a separate thread
	 */
	public MethodInvocation (Method invokingMethod, Method methodCalled, boolean priv, boolean thread){
		this.methodCalled = methodCalled;
		this.invokingMethod = invokingMethod;
		this.priv = priv;
		this.thread = thread;
		methodCalled.addMethodInvocationIn(this);
		invokingMethod.addMethodInvocationOut(this);
	}

	/**
	 * Get the method that would be called
	 * @return
	 */
	public Method getMethodCalled() {
		return methodCalled;
	}

	/**
	 * Set the method that would be called
	 * @param method
	 */
	public void setMethodCalled(Method method) {
		this.methodCalled = method;
	}

	/** True if the method call is privileged */
	public boolean isPriv() {
		return priv;
	}

	/**
	 * Set to true if the method call is privileged, false otherwise 
	 * @param priv
	 */
	public void setPriv(boolean priv) {
		this.priv = priv;
	}

	/** True if the method call is a separate thread */
	public boolean isThread() {
		return thread;
	}

	/**
	 * Set to true if the method call is a separate thread, false otherwise 
	 * @param priv
	 */
	public void setThread(boolean thread) {
		this.thread = thread;
	}

	public Method getInvokingMethod() {
		return invokingMethod;
	}

	public void setInvokingMethod(Method invokingMethod) {
		this.invokingMethod = invokingMethod;
	}
}
