package securityanalyzer.analysismodel;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * Describes a permission without actually making a Permission object
 *
 */
public class PermissionDescription {

	/** the class of the permission */
	private String permissionClass;
	
	/** the arguments passed to this permission */
	private String[] args;
	
	private List<PermissionRequirement> source = new ArrayList<PermissionRequirement>();
	
	/**
	 * Constructor
	 * @param permission - the type of the permission
	 * @param args - the arguments of the permission (empty array if unknown)
	 */
	public PermissionDescription(String permission, String[] args){
		this.permissionClass = permission;
		this.args = args;
	}


	/**
	 * gets the Class of the permission
	 * @return the Class of the permission
	 */
	public String getPermissionClass() {
		return permissionClass;
	}

	/**
	 * gets the name of the permission
	 * @return the name of the permission
	 */
	public String[] getPermissionArgs() {
		return args;
	}

	/**
	 * Get all sources of this permission
	 * @return
	 */
	public List<PermissionRequirement> getSources() {
		return source;
	}

	/**
	 * Add a source of this permission
	 * @param pr
	 */
	public void addSource(PermissionRequirement pr) {
		this.source.add(pr);
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof PermissionDescription)) return false;
		PermissionDescription that = (PermissionDescription) obj;
		
		if (this.args.length != that.args.length) return false;
		
		int i = 0;
		for (String s : args){
			if (!s.equals(that.args[i])) return false;
			i++;
		}
		
		return this.permissionClass.equals(that.permissionClass);
	}
	
	@Override
	public int hashCode() {
		String hc = permissionClass;
		for (String s : args){
			hc += s;
		}
		return hc.hashCode();
	}
}
