package securityanalyzer.resourcemodel;

import java.util.Map;

import java.util.Map.Entry;
import securityanalyzer.resourcemodel.ActionDefinition.AppliedRule;

/**
 * 
 * An instance of an Action. Includes the ability to define parameter values.
 * 
 */
public class FilledActionDefinition extends FilledParameterContainer<ActionDefinition> {

    /**
     * Constructor
     * 
     * @param ad
     *            The ActionDefinition to base this container on
     */
    public FilledActionDefinition(ActionDefinition ad) {
        super(ad);
    }

    /**
     * Resolve this FilledActionDefinition to the appropriate FilledRuleDefinitions
     * @return
     */
    public FilledRuleDefinitionCollection resolveToRuleDefinitions(String ruleType) {
        FilledRuleDefinitionCollection ret = new FilledRuleDefinitionCollection();
        //mappings of our var names to their values
        Map<String, String> mappings =
                VariableResolution.simplifyDefinitionValueMappings(this.getAllMappingsResolveDefaults());

        for (AppliedRule pa : base.getParameterApplications(ruleType)) {
            //every applied rule will create one new rule definition
            FilledRuleDefinition frd = new FilledRuleDefinition(pa.ruleDefinition);
            for (ParameterDefinition b : frd.getAllParameters()) {
                String paramName = b.getName();
                String paramTemplate = pa.nameValueMap.get(paramName);
                String resolvedVal = VariableResolution.resolveVariables(paramTemplate, mappings);
                //drop in the resolved value
                frd.setMapping(b, resolvedVal);
            }
            ret.add(frd);
        }

        return ret;
    }

    public boolean equals(Object o) {
        if (o == null) {
            return false;
        }

        if (!(o instanceof FilledActionDefinition)) {
            return false;
        }

        FilledActionDefinition fad = (FilledActionDefinition) o;

        if (fad.base != base) {
            return false;
        }

        for (Entry<ParameterDefinition, String> e : fad.getAllDefinedMappings().entrySet()) {
            if (!this.getMapping(e.getKey()).equals(e.getValue())) {
                return false;
            }
        }

        return true;
    }
}
