package securityanalyzer.resourcemodel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import securityanalyzer.resourcemodel.ActionDefinition.AppliedRule;

public class FilledRuleDefinitionCollection
        extends FilledParameterContainerCollection<RuleDefinition, FilledRuleDefinition> {

    /**
     * Keeps track of which FilledRuleDefinitions map back to
     * FilledActionDefinitions
     */
    List<FilledRuleDefinition> used = new ArrayList<FilledRuleDefinition>();

    public boolean isUsed(FilledRuleDefinition frd) {
        return used.contains(frd);
    }

    public List<FilledRuleDefinition> getUnused() {
        List<FilledRuleDefinition> ret = new ArrayList<FilledRuleDefinition>();

        for (FilledRuleDefinition frd : this) {
            if (!isUsed(frd)) {
                ret.add(frd);
            }
        }

        return ret;
    }

    private FilledActionDefinitionCollection resolveToActionsOld() {

        FilledActionDefinitionCollection ret = new FilledActionDefinitionCollection();
        for (FilledRuleDefinition frd : this) {
            String type = frd.getBaseObject().getType();
            // Iterate over a list of all actions that require this rule
            for (ActionDefinition ad : frd.getBaseObject().getRequiringActions()) {
                List<ParameterDefinition> actionParams = ad.getParameters();
                // The fAD that we will attempt to build from this action
                // definition
                FilledActionDefinition fad = new FilledActionDefinition(ad);
                // Now we look at all AppliedRules of this type in this action
                // definition
                for (AppliedRule ar : ad.getParameterApplications(type)) {

                    Map<String, String> appliedVars = ar.nameValueMap;

                    // Now for each parameter in the actual filled rule (these
                    // should be filled since we're starting with filledRules
                    // and working backward
                    for (Entry<ParameterDefinition, String> e : frd.getAllDefinedMappings().entrySet()) {
                        // This will get the template value for the given
                        // variable
                        String template = appliedVars.get(e.getKey().getName());
                        String realParameter = e.getValue();

                        // Get the values for the action parameters
                        Map<String, String> actionParamValues = VariableResolution.getActionParametersFromRuleParameter(template,
                                realParameter);

                        // Now build a filledActionDefinition from these reverse
                        // mapped
                        // values

                        for (ParameterDefinition pd : actionParams) {
                            // This param might not be used by this rule
                            String possibleValue = actionParamValues.get(pd.getName());
                            if (possibleValue != null) {
                                fad.setMapping(pd, possibleValue);
                            }
                        }
                    }

                }

                boolean contained = false;
                // Check if this would in fact resolve back down to the same frd
                Set<FilledRuleDefinition> frds = VariableResolution.getFilledRulesFrom(fad, type);
                for (FilledRuleDefinition frdTest : frds) {
                    if (frdTest.equals(frd)) {
                        contained = true;
                    }
                }
                if (contained) {
                    ret.add(fad);
                }
            }
        }
        return ret;
    }

    /**
     * 
     * @param type
     * @return
     */
    public FilledActionDefinitionCollection resolveToActions(String type) {
        
        used = new ArrayList<FilledRuleDefinition>();

        FilledActionDefinitionCollection ret = new FilledActionDefinitionCollection();

        // Get a set of all action definitions
        Set<ResourceDefinition> rds = ResourceModel.getAllResources();

        Set<ActionDefinition> ads = new HashSet<ActionDefinition>();

        for (ResourceDefinition rd : rds) {
            ads.addAll(rd.getActions());
        }

        // Iterate over action definitions
        for (ActionDefinition ad : ads) {
            Map<AppliedRule, Map<FilledRuleDefinition, Map<String, String>>> appliedRuleStore = new HashMap<AppliedRule, Map<FilledRuleDefinition, Map<String, String>>>();
            // For each application of a rule in this action definition
            for (AppliedRule ap : ad.getParameterApplications(type)) {

                // These are things of the form host="{hostname}:[{portrange}]"
                Map<String, String> mappingTemplates = ap.nameValueMap;

                Map<FilledRuleDefinition, Map<String, String>> ruleMaps = new HashMap<FilledRuleDefinition, Map<String, String>>();

                for (FilledRuleDefinition frd : getFilledRulesByDefinition(ap.ruleDefinition)) {

                    Map<String, String> reverseMapping = new HashMap<String, String>();
                    boolean badMatch = false;

                    for (Entry<ParameterDefinition, String> e : frd.getAllDefinedMappings().entrySet()) {

                        String template = "";

                        // Get the template for this parameter and the actual
                        // value as specified in the rule
                        if (e.getKey() instanceof ParameterDefinition) {
                            template = mappingTemplates.get(e.getKey().getName());
                        } else {
                            System.out.println(ap.ruleDefinition.name);
                        }
                        String ruleParameter = e.getValue();

                        // Do the reverse mapping
                        Map<String, String> singleReverseMapping = VariableResolution.getActionParametersFromRuleParameter(template,
                                ruleParameter);

                        String checkMap = VariableResolution.resolveVariables(
                                mappingTemplates.get(e.getKey().getName()),
                                singleReverseMapping);

                        if (!checkMap.equals(ruleParameter)) {
                            badMatch = true;
                        }

                        // Add to the complete map for this rule definition
                        reverseMapping.putAll(singleReverseMapping);
                    }

                    // Now we have a map of all action parameters that we found
                    // from *this* filled rule, and we can add it if there were
                    // no bad matches
                    if (!badMatch) {
                        ruleMaps.put(frd, reverseMapping);
                    }
                }

                // Now we know about all the mappings from rule definitions to
                // string pairs for this applied rule
                appliedRuleStore.put(ap, ruleMaps);
            }

            // Get and store the filled action definitions that result from
            // these mappings and rules
            if (!appliedRuleStore.isEmpty()) {
                ret.addAll(getFilledActionDefinitions(ad, appliedRuleStore));
            }
        }

        return ret;
    }

    private FilledActionDefinitionCollection getFilledActionDefinitions(
            ActionDefinition ad,
            Map<AppliedRule, Map<FilledRuleDefinition, Map<String, String>>> appliedRuleStore) {

        FilledActionDefinitionCollection ret = new FilledActionDefinitionCollection();



        List<Map<FilledRuleDefinition, Map<String, String>>> ruleMaps = new ArrayList<Map<FilledRuleDefinition, Map<String, String>>>();

        ruleMaps.addAll(appliedRuleStore.values());

        List<Map<FilledRuleDefinition, Map<String, String>>> results = getActionRuleSets(ruleMaps);

        // Now go through the complete sets we created and make filled action
        // definitions with the appropriate mappings, and add them to the
        // collection
        for (Map<FilledRuleDefinition, Map<String, String>> ruleMapping : results) {

            // It would be nice if this check wasn't needed, but it seems like
            // getActionRuleSets can return incomplete sets (but never complete
            // ones with wrong entries)
            if (ruleMapping.size() != appliedRuleStore.size()) {
                continue;
            }

            // Make sure to mark the used filled rules
            used.addAll(ruleMapping.keySet());

            // Now make the action definition
            FilledActionDefinition fad = new FilledActionDefinition(ad);

            for (Map<String, String> params : ruleMapping.values()) {

                for (Entry<String, String> e : params.entrySet()) {
                    ParameterDefinition pd = ad.getParameter(e.getKey());
                    // Some error cases
                    if (pd == null) {
                        System.err.println("Parameter [" + e.getKey() + "] does not exist in ActionDefinition [" + ad.getName() + "]");
                    } else if (e.getValue() == null) {
                        System.err.println("Parameter [" + e.getKey() + "] was null for ActionDefinition [" + ad.getName() + "]");
                    } else {
                        fad.setMapping(pd, e.getValue());
                    }
                }

            }

            // Now we can add the created FilledActionDefinition to our
            // collection to return
            ret.add(fad);

        }

        // return the result for this ActionDefinition
        return ret;

    }

    /**
     * Checks the map of parameters to test against all parameters in the given
     * set of rules. Conflict is defined as two parameters A and B where
     * B.name.equals(A) and !B.value.equals(A). In this case, will return false,
     * otherwise true
     * 
     * @param testParams
     * @param existingRuleSet
     * @return false if a conflict is found, true otherwise
     */
    private boolean noParameterConflict(Map<String, String> testParams,
            Map<FilledRuleDefinition, Map<String, String>> existingRuleSet) {

        // Go through all rule-parameter mappings already found and ensure no
        // parameter conflicts
        for (Map<String, String> params : existingRuleSet.values()) {
            for (Entry<String, String> testEntry : testParams.entrySet()) {
                for (Entry<String, String> entry : params.entrySet()) {
                    String name = entry.getKey(), testName = testEntry.getKey(), value = entry.getValue(), testValue = testEntry.getValue();

                    // A conflict is two params with the same name having
                    // different values
                    if (name.equals(testName) && !value.equals(testValue)) {
                        return false;
                    }
                }
            }
        }

        return true;

    }

    /**
     * Gets a List of all filled rules that have the given RuleDefinition as a
     * base
     * 
     * @param rd
     * @return
     */
    List<FilledRuleDefinition> getFilledRulesByDefinition(RuleDefinition rd) {

        List<FilledRuleDefinition> frds = new ArrayList<FilledRuleDefinition>();

        for (FilledRuleDefinition frd : this) {
            if (frd.getBaseObject().equals(rd)) {
                frds.add(frd);
            }
        }

        return frds;

    }

    /**
     * Returns a list of maps of rules to their parameters. Each list is a set
     * of filled rules from as many different applied rules as possible with no
     * conflicting parameters. The maps that have the same size as the initial
     * size of the rules parameter have one filled rule for each applied rule,
     * and represent a complete filled action.
     * 
     * The types in this method are just a little ridiculous and should probably
     * be refactored into private classes
     * 
     * @param rules
     *            A list of maps of filled rules to their parameters, each map
     *            coming from one applied rule
     * @return
     */
    private List<Map<FilledRuleDefinition, Map<String, String>>> getActionRuleSets(
            List<Map<FilledRuleDefinition, Map<String, String>>> rules) {

        Map<FilledRuleDefinition, Map<String, String>> start = new HashMap<FilledRuleDefinition, Map<String, String>>();

        List<Map<FilledRuleDefinition, Map<String, String>>> ret = getActionRuleSets0(
                start, rules);

        // If null was returned, there were no matches whatsoever
        if (ret == null) {
            return new ArrayList<Map<FilledRuleDefinition, Map<String, String>>>();
        }

        return ret;
    }

    /**
     * Returns a list of maps of rules to their parameters. Each list is a set
     * of filled rules from as many different applied rules as possible with no
     * conflicting parameters. The maps that have the same size as the initial
     * size of the rules parameter have one filled rule for each applied rule,
     * and represent a complete action.
     * 
     * The types in this method are just a little ridiculous and should probably
     * be refactored into private classes
     * 
     * @param soFar
     *            Should be empty to start (used to hold data recursively)
     * @param rules
     *            A list of maps of filled rules to their parameters, each map
     *            coming from one applied rule
     * @return
     */
    private List<Map<FilledRuleDefinition, Map<String, String>>> getActionRuleSets0(
            Map<FilledRuleDefinition, Map<String, String>> soFar,
            List<Map<FilledRuleDefinition, Map<String, String>>> rules) {

        List<Map<FilledRuleDefinition, Map<String, String>>> ret = new ArrayList<Map<FilledRuleDefinition, Map<String, String>>>();

        // Base case of success (made it to the bottom): return an empty list
        if (rules.isEmpty()) {
            return ret;
        }

        // Get and the first mapping in the passed list so we're now looking at
        // all filled rules and their parameters from the first relevant applied
        // rule
        Map<FilledRuleDefinition, Map<String, String>> currentRules = rules.get(0);

        // Now make a shallow copy of the rules and remove the first element.
        // Need to copy this so we don't edit the reference to rules and remove
        // data from other branches.
        List<Map<FilledRuleDefinition, Map<String, String>>> restOfRules = new ArrayList<Map<FilledRuleDefinition, Map<String, String>>>();
        restOfRules.addAll(rules);
        restOfRules.remove(0);

        // Now look at each of these rules
        for (Entry<FilledRuleDefinition, Map<String, String>> e : currentRules.entrySet()) {

            // Make a shallow copy of the filled rules we've seen so far (we
            // won't edit this data, but we want copies of references)
            Map<FilledRuleDefinition, Map<String, String>> newSoFar = new HashMap<FilledRuleDefinition, Map<String, String>>();
            newSoFar.putAll(soFar);

            // Check if the current filled rule's parameter mappings conflict with any before us
            if (!noParameterConflict(e.getValue(), soFar)) {
                // The failure case - there was a parameter conflict so return null for this branch
                return null;
            } else {

                // Record that this is now part of the set we're considering
                newSoFar.put(e.getKey(), e.getValue());

                // Find out what comes "after" this
                List<Map<FilledRuleDefinition, Map<String, String>>> belowThis = getActionRuleSets0(
                        newSoFar, restOfRules);

                // Now there's three cases to handle - failure, the base case,
                // and the normal case

                // 1.  If the search failed from here, propagate the failure case of
                // null back up the chain
                if (belowThis == null) {
                    return null;
                }

                // 2.  If the search had results from here, add our current filled
                // rule of interest to each of those maps and add the maps to this
                // call's eventual return value
                for (Map<FilledRuleDefinition, Map<String, String>> tail : belowThis) {
                    tail.put(e.getKey(), e.getValue());
                    ret.add(tail);
                }

                // 3.  If the search hit the base case, then just make a new, empty
                // map to start adding to as the recursion unwinds
                if (belowThis.isEmpty()) {
                    Map<FilledRuleDefinition, Map<String, String>> startOfSet = new HashMap<FilledRuleDefinition, Map<String, String>>();
                    startOfSet.put(e.getKey(), e.getValue());
                    ret.add(startOfSet);
                }
            }

        }

        // Finally, return our list of maps
        return ret;

    }
}
