package securityanalyzer.resourcemodel;

/**
 * 
 * Defines a parameter.
 * 
 */
public class ParameterDefinition extends Definition {
	
	/**
	 * The type contained by this parameter. The Java type is always String, but
	 * can have significantly different forms - such as file references, or host
	 * addresses
	 */
	private String type;
	/**
	 * If no value is specified for this parameter, this default will be used.
	 * If the default is 'null', this parameter is required.
	 */
	private String defaultVal;
	private int id;

	/**
	 * Constructor
	 * 
	 * @param name
	 *            The name of this parameter
	 * @param type
	 *            The type of this parameter
	 * @param defaultVal
	 *            The default value of this parameter (null if no default)
	 * @param description
	 *            The description of this parameter
	 */
	public ParameterDefinition(String name, String type, String defaultVal,
			String description, int id) {
		this.name = name;
		this.type = type;
		this.defaultVal = defaultVal;
		this.description = description;
		this.id = id;
	}

	/**
	 * Constructor
	 * 
	 * @param name
	 *            The name of this parameter
	 * @param type
	 *            The type of this parameter
	 * @param defaultVal
	 *            The default value of this parameter (null if no default)
	 */
	public ParameterDefinition(String name, String type, String defaultVal) {
		this.name = name;
		this.type = type;
		this.defaultVal = defaultVal;
	}

	/**
	 * Constructor
	 * 
	 * @param name
	 *            The name of this parameter
	 * @param type
	 *            The type of this parameter
	 */
	public ParameterDefinition(String name, String type) {
		this.name = name;
		this.type = type;
		this.defaultVal = null;
	}

	/**
	 * Get the type of this parameter
	 * 
	 * @return
	 */
	public String getType() {
		return type;
	}

	/**
	 * Get the default value of this parameter
	 * 
	 * @return The default value, or 'null' if there isn't one
	 */
	public String getDefaultVal() {
		return defaultVal;
	}

	/**
	 * Set the default value
	 * 
	 * @param defaultVal
	 */
	public void setDefaultVal(String defaultVal) {
		this.defaultVal = defaultVal;
	}

	/**
	 * Set the type of this parameter
	 * 
	 * @param type
	 */
	public void setType(String type) {
		this.type = type;
	}

	public int getId() {
		return this.id;
	}
}
