package securityanalyzer.resourcemodel;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import securityanalyzer.analysismodel.PermissionDescription;

public class PermissionConverter implements PolicySpecificConverter<PermissionDescription> {

	public PermissionDescription convertToPolicySpecificObject(
			FilledRuleDefinition frd) {
		if (frd == null) return null;
		
	    Map<ParameterDefinition, String> params = frd.getAllMappingsResolveDefaults();
	    List<ParameterDefinition> sortedParamDefs = frd.getBaseObject().getParameters();
	    
	    String[] args = new String[sortedParamDefs.size()];
	    int i = 0;
	    for (ParameterDefinition pd : sortedParamDefs){
	    	args[i] = params.get(pd);
	    	i++;
	    }
	    
		return new PermissionDescription(
				frd.getBaseObject().getAttribute("classname"), args);
	}

	public Collection<PermissionDescription> convertToPolicySpecificObjects(
			FilledRuleDefinitionCollection frdc) {
		Collection<PermissionDescription> pds = new HashSet<PermissionDescription>();
		if (frdc == null) return pds;
		for (FilledRuleDefinition frd : frdc){
			pds.add(convertToPolicySpecificObject(frd));
		}
		return pds;
	}
	
	public FilledRuleDefinition convertToRule(PermissionDescription policyObject) {
		if (policyObject == null) return null;
		RuleDefinition prd = null;
		for (RuleDefinition rd : ResourceModel.getAllRules()){
			if (rd.getAttribute("classname") 
			.equals(policyObject.getPermissionClass())){
				prd = rd;
				break;
			}
		}
		if (prd == null){
			//TODO handle missing RuleDefinitions
			return null;
		}
			
		String[] realArgs = policyObject.getPermissionArgs();
		List<ParameterDefinition> abstractArgDefs = prd.getParameters();
		
		FilledRuleDefinition pi = new FilledRuleDefinition(prd);
		
		for (ParameterDefinition p : abstractArgDefs){
			pi.setMapping(p, realArgs[p.getId() - 1]);
		}
		return pi;

	}
	
	public FilledRuleDefinitionCollection convertToRules(
			Collection<PermissionDescription> policyObjects) {
		
		FilledRuleDefinitionCollection ret = new FilledRuleDefinitionCollection();
		if (policyObjects == null) return ret;
		for (PermissionDescription p : policyObjects) {
			ret.add(convertToRule(p));
		}
		return ret;
	}
}
