package securityanalyzer.resourcemodel;

import java.util.Collection;

/**
 * 
 * Classes implementing this interface are able to resolve a
 * {@link FilledActionDefinition} to some set of objects that represent the
 * application of the defined policy in a specific security system
 * 
 * @param <T>
 *            The type of objects that may appear in the resolved set
 */
public interface PolicySpecificConverter<T> {

	/**
	 * Convert {@link FilledRuleDefinition} into a policy specific object
	 * 
	 * @param frd
	 * @return
	 */
	public T convertToPolicySpecificObject(FilledRuleDefinition frd);
	
	/**
	 * Resolve the given {@link FilledRuleDefinitionCollection} to some set of
	 * objects that represent the application of the defined policy in a
	 * specific security system
	 * 
	 * @param fad
	 * @return
	 */
	public Collection<T> convertToPolicySpecificObjects(FilledRuleDefinitionCollection frdc);

	/**
	 * Convert the policy specific object into the proper
	 * {@link FilledRuleDefinition}
	 * 
	 * @param policyObject
	 * @return
	 */
	public FilledRuleDefinition convertToRule(T policyObject);
	
	/**
	 * Convert policy specific objects into general {@link FilledRuleDefinition}
	 * s
	 * 
	 * @param policyObjects
	 * @return
	 */
	public FilledRuleDefinitionCollection convertToRules(Collection<T> policyObjects);

}
