package securityanalyzer.resourcemodel;

import java.util.HashSet;
import java.util.Set;

/**
 * 
 * Defines a Resource. Resources are containers of actions
 * 
 */
public class ResourceDefinition extends Definition {
	/** All actions that can be performed on this resource */
	private Set<ActionDefinition> actions;

	/**
	 * Constructor
	 * 
	 * @param name
	 *            the name of this resource
	 * @param description
	 *            the description of this resource
	 */
	public ResourceDefinition(String name, String description) {
		this.description = description;
		this.name = name;
		this.actions = new HashSet<ActionDefinition>();
	}

	/**
	 * Add an action to this resource
	 * 
	 * @param a
	 *            Action to add
	 * @return true if the action wasn't already part of this resource
	 */
	public boolean addAction(ActionDefinition a) {
		return actions.add(a);
	}

	/**
	 * Get all possible actions for this resource
	 * 
	 * @return
	 */
	public Set<ActionDefinition> getActions() {
		return actions;
	}
	
	public ActionDefinition getActionByName(String name) {
		for(ActionDefinition ad: actions) {
			if(ad.getName().equals(name)) {
				return ad;
			}
		}
		
		return null;
	}

    @Override
    public int hashCode() {
        String chstr = name + description;
        for (ActionDefinition ad : actions){
            chstr += "" + ad.hashCode();
        }
        return chstr.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof ResourceDefinition)) return false;
        return this.hashCode() == obj.hashCode();
    }
        
    
}
