package securityanalyzer.resourcemodel;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import securityanalyzer.resourcemodel.reader.ResourceModelReader;

/**
 * 
 * Defines a way of talking about rules in terms of Resources and Actions
 * that can be performed on Resources.
 * 
 */
public class ResourceModel {

	/** All rules that we currently know about */
	private static Set<RuleDefinition> rules = new HashSet<RuleDefinition>();
	/** All resources that we currently know about */
	private static Set<ResourceDefinition> resources = new HashSet<ResourceDefinition>();

	/**
	 * No one should be allowed to call the constructor.
	 */
	private ResourceModel() {
	}

	/**
	 * Add rule and resource definitions from an XML file
	 * 
	 * @param xmlFile
	 */
	public static void loadFromXML(String xmlFile) {
            try {
		ResourceModelReader.setResourcesAndPermissions(new File(xmlFile));
            } catch (Exception e) {
                //FIXME : handle parse errors better!!
                System.err.println("Error while parsing: " + xmlFile);
                e.printStackTrace();
            }
	}

	/**
	 * Add a rule definition
	 * 
	 * @param p
	 *            rule definition to add
	 * @return true if the rule was not already known
	 */
	public static boolean addRuleDefinition(RuleDefinition p) {
		return rules.add(p);
	}

	/**
	 * Add a resource
	 * 
	 * @param r
	 *            resource to add
	 * @return true if the resource was not already known
	 */
	public static boolean addResourceDefinition(ResourceDefinition r) {
		return resources.add(r);
	}

	/**
	 * Get all known permissions
	 * 
	 * @return
	 */
	public static Set<RuleDefinition> getAllRules() {
		return rules;
	}

	/**
	 * Get all known resources
	 * 
	 * @return
	 */
	public static Set<ResourceDefinition> getAllResources() {
		return resources;
	}

	public static Set<FilledActionDefinition> determineActionsFromRules(
			Set<FilledRuleDefinition> perms) {
		//TODO write the backwards mapping from permission grants to actions
		return null;
	}
	
	public static RuleDefinition getRuleByName(String name) {

		for (RuleDefinition rd : ResourceModel.getAllRules()) {
			if (rd.getName().equals(name)) {
				return rd;
			}
		}

		return null;
	}
	
	public static ResourceDefinition getResourceByName(String name) {
		for (ResourceDefinition rd : ResourceModel.getAllResources()) {
			if (rd.getName().equals(name)) {
				return rd;
			}
		}

		return null;
	}

}
