package securityanalyzer.resourcemodel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 
 * Defines a rule. Rules contain a set of parameters.
 * 
 */
public class RuleDefinition extends Definition implements
		ParameterContainer {

	/** The string definition of the permission rule type */
	public static final String TYPE_PERMISSION = "Permission";
	
	/** The parameters of this rule */
	private List<ParameterDefinition> parameters;
	/** The type of this rule */
	private String type;
	/** Attribute fields of the rule (includes things like classname for a permission rule) */
	private Map<String, String> attributes;
	/** Actions that require this rule.  Used for reverse mapping.*/
	private List<ActionDefinition> requiringActions;

	/**
	 * Constructor
	 * 
	 * @param name
	 *            The name of this rule
	 * @param description
	 *            The description of this permission
	 * @param type
	 *            The type of this rule
	 */
	public RuleDefinition(String name, String description, String type) {
		this.name = name;
		this.description = description;
		this.type = type;
		this.parameters = new ArrayList<ParameterDefinition>();
		this.attributes = new HashMap<String, String>();
		this.requiringActions = new ArrayList<ActionDefinition>();
	}

	public List<ParameterDefinition> getParameters() {
		return parameters;
	}

	public boolean addParameter(ParameterDefinition p) {
		return parameters.add(p);
	}
	
	public Map<String, String> getAttributes() {
		return attributes;
	}
	
	public String getAttribute(String key) {
		return attributes.get(key);
	}
	
	public String addAttribute(String key, String value) {
		return attributes.put(key, value);
	}
	
	public String getType() {
		return type;
	}
	
	public List<ActionDefinition> getRequiringActions() {
		return requiringActions;
	}
	
	public boolean addRequiringAction(ActionDefinition ad) {
		return requiringActions.add(ad);
	}
	
	public ParameterDefinition getParameterByName(String name) {
		for(ParameterDefinition pd: parameters) {
			if(pd.getName().equals(name)) {
				return pd;
			}
		}
		
		return null;
	}
}
