package securityanalyzer.resourcemodel;

import java.io.File;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import securityanalyzer.analysismodel.PermissionDescription;
import securityanalyzer.resourcemodel.reader.ResourceModelReader;

public class PermissionConverterTest {
	
	private List<PermissionDescription> pds;
	private List<FilledRuleDefinition> frds;
	private PermissionConverter converter;
	
	@Before
	public void setup(){
		ResourceModelReader.setResourcesAndPermissions(new File("Resource Model/secondDraft.xml"));
		converter = new PermissionConverter();
		pds = new LinkedList<PermissionDescription>();
		String[] ps1 = {
				"/path/to/files", "read"
		};
		pds.add(new PermissionDescription("java.io.FilePermission", ps1));
		
		String[] ps2 = {
				"*.wpi.edu", "connect"
		};
		pds.add(new PermissionDescription("java.net.SocketPermission", ps2));
		
		String[] ps3 = {
				"/more/files", "write"
		};
		pds.add(new PermissionDescription("java.io.FilePermission", ps3));
		
		
		frds = new LinkedList<FilledRuleDefinition>();
		FilledRuleDefinition frd = new FilledRuleDefinition(ResourceModel.getRuleByName("FilePermission"));
		for (ParameterDefinition p : frd.getUnmappedParameters()){
			if (p.getName().equals("path")){
				frd.setMapping(p, "/path/to/files");
			} else if (p.getName().equals("action")) {
				frd.setMapping(p, "read");
			} else {
				System.err.println("Unexpected parameter:" + p.getName());
			}
		}
		frds.add(frd);
		
		frd = new FilledRuleDefinition(ResourceModel.getRuleByName("SocketPermission"));
		for (ParameterDefinition p : frd.getUnmappedParameters()){
			if (p.getName().equals("host")){
				frd.setMapping(p, "*.wpi.edu");
			} else if (p.getName().equals("action")) {
				frd.setMapping(p, "connect");
			} else {
				System.err.println("Unexpected parameter:" + p.getName());
			}
		}
		frds.add(frd);
		
		frd = new FilledRuleDefinition(ResourceModel.getRuleByName("FilePermission"));
		for (ParameterDefinition p : frd.getUnmappedParameters()){
			if (p.getName().equals("path")){
				frd.setMapping(p, "/more/files");
			} else if (p.getName().equals("action")) {
				frd.setMapping(p, "write");
			} else {
				System.err.println("Unexpected parameter:" + p.getName());
			}
		}
		frds.add(frd);
	}

	@Test
	public void testConvertToPolicySpecificObject() {
		for (int i = 0; i < pds.size(); i++){
			Assert.assertEquals(pds.get(i), converter
					.convertToPolicySpecificObject(frds.get(i)));
		}
		
		Assert.assertNull("null input did not return null", converter.convertToPolicySpecificObject(null));
	}

	@Test
	public void testConvertToPolicySpecificObjects() {
		FilledRuleDefinitionCollection frdc = new FilledRuleDefinitionCollection();
		frdc.addAll(frds);
		Collection<PermissionDescription> gpds = converter.convertToPolicySpecificObjects(frdc);
		
		Assert.assertEquals("number of generated policy objects is incorrect", pds.size(), gpds.size());
		for (PermissionDescription p : pds){
			Assert.assertTrue(gpds.contains(p));
		}
		
		//running size() makes sure a) we don't get null, and b) we get an empty set
		Assert.assertEquals(0, converter.convertToPolicySpecificObjects(null).size());
	}

	@Test
	public void testConvertToRule() {
		for (int i = 0; i < pds.size(); i++){
			Assert.assertEquals(frds.get(i), converter
					.convertToRule(pds.get(i)));
		}
		
		Assert.assertNull("null input did not return null", converter.convertToRule(null));
	}

	@Test
	public void testConvertToRules() {
		FilledRuleDefinitionCollection frdc = converter.convertToRules(pds);
		Assert.assertEquals("number of generated FRDs is incorrect", frds.size(), frdc.size());
		
		for (FilledRuleDefinition frd : frdc){
			Assert.assertTrue(frds.contains(frd));
		}
		
		//running size() makes sure a) we don't get null, and b) we get an empty set
		Assert.assertEquals(0, converter.convertToRules(null).size());
	}

}
