package securityanalyzer.resourcemodel;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.util.HashMap;
import java.util.Map;

import org.junit.Test;


public class VariableResolutionTest {
	
	@Test
	public void easyTest() {
		
		String value = "{1}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"hello!");
		
	}
	
	@Test
	public void sameIDTwiceTest() {
		
		String value = "{1} {1}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"hello! hello!");
		
	}
	
	@Test
	public void twoIDsTest() {
		
		String value = "{1} {2}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		mappings.put("2", "goodbye!");
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"hello! goodbye!");
		
	}
	
	@Test
	public void twoIDsTest2() {
		
		String value = "{1}{2}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		mappings.put("2", "goodbye!");
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"hello!goodbye!");
		
	}
	
	@Test
	public void emptyReplace() {
		
		String value = "{1}{2}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		mappings.put("2", "");
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"hello!");
		
	}
	
	@Test
	public void nullReplace() {
		
		String value = "{2}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("2", null);
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"");
		
	}
	
	@Test
	public void nestedBrackets() {
		
		//TODO: Look at this one
		String value = "{1{}2}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		mappings.put("2", "goodbye!");
		
		assertNull(VariableResolution.resolveVariables(value, mappings));
		
	}
	
	@Test
	

	public void mismatchedBracketsOpen() {
		
		String value = "{{1}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		mappings.put("2", "goodbye!");
		
		assertNull(VariableResolution.resolveVariables(value, mappings));

		

	}
	
	@Test
	public void mismatchedBracketsClosed() {
		
		String value = "{1}}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("1", "hello!");
		mappings.put("2", "goodbye!");
		
		assertNull(VariableResolution.resolveVariables(value, mappings));
		
	}
	
	@Test
	public void noMappings() {
		
		String value = "{var}";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"");
		
	}
	
	@Test
	public void surroundedText() {
		
		String value = "Hel{var}lo";
		
		Map<String, String> mappings = new HashMap<String, String>();
		
		mappings.put("var", "");
		
		assertEquals(VariableResolution.resolveVariables(value, mappings),"Hello");
		
	}
	
}
