package securityanalyzer.resourcemodel.reader;

import securityanalyzer.resourcemodel.ActionDefinition;
import securityanalyzer.resourcemodel.ParameterDefinition;
import securityanalyzer.resourcemodel.ResourceDefinition;
import securityanalyzer.resourcemodel.ResourceModel;
import securityanalyzer.resourcemodel.RuleDefinition;
import securityanalyzer.resourcemodel.ActionDefinition.AppliedRule;

public class DefinitionDumper {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		if (args.length != 1) {
			usage();
			System.exit(1);
		}

		try {
			ResourceModel.loadFromXML(args[0]);
		} catch (Exception e) {
			System.out.println("Problem reading file " + args[0]
					+ " or maybe something else.");
			e.printStackTrace();
		}

		System.out.println("------------------------------------------------");
		System.out.println("Resources:");
		System.out.println("------------------------------------------------");
		for (ResourceDefinition r : ResourceModel.getAllResources()) {
			System.out.println(r.getName());
			System.out.println("  Actions:");
			for (ActionDefinition a : r.getActions()) {
				System.out.println("    " + a.getName());
				System.out.println("      ActionParameters:");
				for (ParameterDefinition pd : a.getParameters()) {
					System.out.println("        " + pd.getName());
				}
				System.out.println("      RuleDefinitions:");

				for (AppliedRule ar : a.getParameterApplications("Permission")) {
					System.out.print("        " + ar.ruleDefinition.getName()
							+ "(");
					for (java.util.Map.Entry<String, String> e : ar.nameValueMap
							.entrySet()) {
						System.out.print("[" + e.getKey() + ":" + e.getValue()
								+ "] ");
					}
					System.out.println(")");
				}
			}
		}
		System.out
				.println("\n------------------------------------------------");
		System.out.println("Rules:");
		System.out.println("------------------------------------------------");
		for (RuleDefinition rd : ResourceModel.getAllRules()) {
			System.out.println(rd.getName());
			System.out.println("  Parameters:");
			for (ParameterDefinition pd : rd.getParameters()) {
				System.out.println("    " + pd.getId() + ": " + pd.getName());
			}
			System.out.println(rd.getDescription());

		}

	}

	private static void usage() {

		System.out
				.println("Takes a filename of an instance of resourcetorules.xsd.");

	}

}
